﻿using System;
using System.Diagnostics.Contracts;
using System.Reactive.Linq;

namespace Rxx.Labs
{
	internal static class ObservableExtensions
	{
		public static IObservable<T> Do<T>(this IObservable<T> source, Func<IObserver<object>> observerFactory)
		{
			Contract.Requires(observerFactory != null);
			Contract.Ensures(Contract.Result<IObservable<T>>() != null);

			Contract.Assume(source != null);

			return source.Do(new TypeCoercingObserver<T, object>(observerFactory()));
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes", 
			Justification = "Exception is passed to the observer.")]
		public static void ForEach<T>(this IObservable<T> source, Func<IObserver<object>> observerFactory)
		{
			Contract.Requires(observerFactory != null);

			Contract.Assume(source != null);

			var observer = new TypeCoercingObserver<T, object>(observerFactory());

			try
			{
				source.ForEach(observer.OnNext);
			}
			catch (Exception ex)
			{
				observer.OnError(ex);
				return;
			}

			observer.OnCompleted();
		}

		public static IDisposable Subscribe<T>(this IObservable<T> source, Func<IObserver<object>> observerFactory)
		{
			Contract.Requires(observerFactory != null);
			Contract.Ensures(Contract.Result<IDisposable>() != null);

			Contract.Assume(source != null);

			var observer = new TypeCoercingObserver<T, object>(observerFactory());

			return source.Subscribe(observer);
		}
	}
}
