﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reactive.Linq;
using System.Reactive.Subjects;
using Rxx.Labs.Properties;

namespace Rxx.Labs.Reactive.UI
{
	[DisplayName("View Model")]
	[Description("Attaching a view model to a user interface.")]
	public partial class ViewModelLab : BaseLab
	{
		public ViewModelLab()
		{
			InitializeComponent();
		}
	}

	public sealed class LabViewModel : Rxx.ViewModel
	{
		public string Words
		{
			get;
			set;
		}

		public CommandSubject Submit
		{
			get
			{
				return submit;
			}
		}

		public IObservable<string> WordsEnteredAndCounter
		{
			get
			{
				return wordsEnteredAndCounter;
			}
		}

		private readonly CommandSubject submit = new CommandSubject();
		private readonly Subject<string> wordsEnteredAndCounter = new Subject<string>();

		public LabViewModel()
		{
			Words = Text.EnterWordsHere;
		}

		protected override IEnumerable<IDisposable> Attaching()
		{
			if (!IsInDesignMode)
			{
				((BaseLab) Element).TraceLine(Text.AttachingFormat, DateTime.Now.TimeOfDay);
			}

			yield return submit
				.Select(_ => Observable.Interval(TimeSpan.FromSeconds(1)).Select(i => i + " - " + Words))
				.Switch()
				.Subscribe(wordsEnteredAndCounter);
		}

		protected override void Detaching()
		{
			((BaseLab) Element).TraceLine(Text.DetachingFormat, DateTime.Now.TimeOfDay);
		}
	}
}