﻿using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Reactive.Joins;

namespace System.Reactive.Linq
{
	public static partial class Observable
	{
		public static IObservable<TSource> Aggregate<TSource>(this IObservable<TSource> source, Func<TSource, TSource, TSource> accumulator)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Requires(accumulator != null, null, "accumulator != null");
			Contract.Ensures(Contract.Result<IObservable<TSource>>() != null, null, "Contract.Result<IObservable<TSource>>() != null");
			return null;
		}

		public static IObservable<TAccumulate> Aggregate<TSource, TAccumulate>(this IObservable<TSource> source, TAccumulate seed, Func<TAccumulate, TSource, TAccumulate> accumulator)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Requires(accumulator != null, null, "accumulator != null");
			Contract.Ensures(Contract.Result<IObservable<TSource>>() != null, null, "Contract.Result<IObservable<TSource>>() != null");
			return null;
		}

		public static IObservable<bool> All<TSource>(this IObservable<TSource> source, Func<TSource, bool> predicate)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Requires(predicate != null, null, "predicate != null");
			Contract.Ensures(Contract.Result<IObservable<TSource>>() != null, null, "Contract.Result<IObservable<TSource>>() != null");
			return null;
		}

		public static IObservable<TSource> Amb<TSource>(params IObservable<TSource>[] sources)
		{
			Contract.Requires(sources != null, null, "sources != null");
			//Contract.Requires(Contract.ForAll(sources, observable => observable != null), null, "Contract.ForAll(sources, observable => observable != null)");
			Contract.Ensures(Contract.Result<IObservable<TSource>>() != null, null, "Contract.Result<IObservable<TSource>>() != null");
			return null;
		}

		public static IObservable<TSource> Amb<TSource>(this IEnumerable<IObservable<TSource>> sources)
		{
			Contract.Requires(sources != null, null, "sources != null");
			//Contract.Requires(Contract.ForAll(sources, observable => observable != null), null, "Contract.ForAll(sources, observable => observable != null)");
			Contract.Ensures(Contract.Result<IObservable<TSource>>() != null, null, "Contract.Result<IObservable<TSource>>() != null");
			return null;
		}

		public static IObservable<TSource> Amb<TSource>(this IObservable<TSource> first, IObservable<TSource> second)
		{
			Contract.Requires(first != null, null, "first != null");
			Contract.Requires(second != null, null, "second != null");
			Contract.Ensures(Contract.Result<IObservable<TSource>>() != null, null, "Contract.Result<IObservable<TSource>>() != null");
			return null;
		}

		public static Pattern<TLeft, TRight> And<TLeft, TRight>(this IObservable<TLeft> left, IObservable<TRight> right)
		{
			Contract.Requires(left != null, null, "left != null");
			Contract.Requires(right != null, null, "right != null");
			Contract.Ensures(Contract.Result<Pattern<TLeft, TRight>>() != null, null, "Contract.Result<Pattern<TLeft, TRight>>() != null");
			return null;
		}

		public static IObservable<bool> Any<TSource>(this IObservable<TSource> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<bool>>() != null, null, "Contract.Result<IObservable<bool>>() != null");
			return null;
		}

		public static IObservable<bool> Any<TSource>(this IObservable<TSource> source, Func<TSource, bool> predicate)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Requires(predicate != null, null, "predicate != null");
			Contract.Ensures(Contract.Result<IObservable<bool>>() != null, null, "Contract.Result<IObservable<bool>>() != null");
			return null;
		}

		public static IObservable<TSource> AsObservable<TSource>(this IObservable<TSource> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<TSource>>() != null, null, "Contract.Result<IObservable<TSource>>() != null");
			return null;
		}

		public static IObservable<decimal?> Average(this IObservable<decimal?> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<decimal?>>() != null, null, "Contract.Result<IObservable<decimal?>>() != null");
			return null;
		}

		public static IObservable<decimal> Average(this IObservable<decimal> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<decimal>>() != null, null, "Contract.Result<IObservable<decimal>>() != null");
			return null;
		}

		public static IObservable<double?> Average(this IObservable<double?> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<double?>>() != null, null, "Contract.Result<IObservable<double?>>() != null");
			return null;
		}

		public static IObservable<double> Average(this IObservable<double> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<double>>() != null, null, "Contract.Result<IObservable<double>>() != null");
			return null;
		}

		public static IObservable<float?> Average(this IObservable<float?> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<float?>>() != null, null, "Contract.Result<IObservable<float?>>() != null");
			return null;
		}

		public static IObservable<float> Average(this IObservable<float> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<float>>() != null, null, "Contract.Result<IObservable<float>>() != null");
			return null;
		}

		public static IObservable<double?> Average(this IObservable<int?> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<double?>>() != null, null, "Contract.Result<IObservable<double?>>() != null");
			return null;
		}

		public static IObservable<double> Average(this IObservable<int> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<double>>() != null, null, "Contract.Result<IObservable<double>>() != null");
			return null;
		}

		public static IObservable<double?> Average(this IObservable<long?> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<double?>>() != null, null, "Contract.Result<IObservable<double?>>() != null");
			return null;
		}

		public static IObservable<double> Average(this IObservable<long> source)
		{
			Contract.Requires(source != null, null, "source != null");
			Contract.Ensures(Contract.Result<IObservable<double>>() != null, null, "Contract.Result<IObservable<double>>() != null");
			return null;
		}
	}
}